@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.42.2.16 2005/08/04 20:45:14 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_dcache.c,v 1.64 2005/10/13 15:12:07 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -23,14 +23,31 @@ RCSID
 #include "afs/afs_osidnlc.h"
 
 /* Forward declarations. */
-static void afs_GetDownD(int anumber, int *aneedSpace);
+static void afs_GetDownD(int anumber, int *aneedSpace, afs_int32 buckethint);
 static void afs_FreeDiscardedDCache(void);
 static void afs_DiscardDCache(struct dcache *);
 static void afs_FreeDCache(struct dcache *);
+/* For split cache */
+static afs_int32 afs_DCGetBucket(struct vcache *);
+static void afs_DCAdjustSize(struct dcache *, afs_int32, afs_int32);
+static void afs_DCMoveBucket(struct dcache *, afs_int32, afs_int32);
+static void afs_DCSizeInit(void);
+static afs_int32 afs_DCWhichBucket(afs_int32, afs_int32);
+
 
 /*
  * --------------------- Exported definitions ---------------------
  */
+/* For split cache */
+afs_int32 afs_blocksUsed_0;    /*1K blocks in cache - in theory is zero */
+afs_int32 afs_blocksUsed_1;    /*1K blocks in cache */
+afs_int32 afs_blocksUsed_2;    /*1K blocks in cache */
+afs_int32 afs_pct1 = -1;
+afs_int32 afs_pct2 = -1;
+afs_uint32 afs_tpct1 = 0;
+afs_uint32 afs_tpct2 = 0;
+afs_uint32 splitdcache = 0;
+
 afs_lock_t afs_xdcache;		/*Lock: alloc new disk cache entries */
 afs_int32 afs_freeDCList;	/*Free list for disk cache entries */
 afs_int32 afs_freeDCCount;	/*Count of elts in freeDCList */
@@ -119,6 +136,112 @@ struct afs_cacheOps afs_MemCacheOps = {
 int cacheDiskType;		/*Type of backing disk for cache */
 struct afs_cacheOps *afs_cacheType;
 
+static afs_int32
+afs_DCGetBucket(struct vcache *avc) 
+{
+  /* This should be replaced with some sort of user configurable function */
+  if (avc->states & CRO) {
+      return 2;
+  } else if (avc->states & CBackup) {
+      return 1;
+  } else {
+    /* RW */
+  }
+  /* main bucket */
+  return 1;
+}
+
+static void 
+afs_DCAdjustSize(struct dcache *adc, afs_int32 oldSize, afs_int32 newSize)
+{
+    afs_int32 adjustSize = newSize - oldSize;
+
+    if (!splitdcache) 
+	return;
+
+    switch (adc->bucket) 
+    {
+    case 0:
+	afs_blocksUsed_0 += adjustSize;
+	afs_stats_cmperf.cacheBucket0_Discarded += oldSize;
+	break;
+    case 1:
+	afs_blocksUsed_1 += adjustSize;
+	afs_stats_cmperf.cacheBucket1_Discarded += oldSize;
+	break;
+    case 2:
+	afs_blocksUsed_2 += adjustSize;
+	afs_stats_cmperf.cacheBucket2_Discarded += oldSize;
+	break;
+    }
+
+    return;
+}
+
+static void 
+afs_DCMoveBucket(struct dcache *adc, afs_int32 size, afs_int32 newBucket)
+{
+    if (!splitdcache) 
+	return;
+
+    switch (adc->bucket) 
+    {
+    case 0:
+	afs_blocksUsed_0 -= size;
+	break;
+    case 1:
+	afs_blocksUsed_1 -= size;
+	break;
+    case 2:
+	afs_blocksUsed_2 -= size;
+	break;
+    }
+
+    adc->bucket = newBucket;
+
+    switch (adc->bucket) 
+    {
+    case 0:
+	afs_blocksUsed_0 += size;
+	break;
+    case 1:
+	afs_blocksUsed_1 += size;
+	break;
+    case 2:
+	afs_blocksUsed_2 += size;
+	break;
+    }
+    
+    return;
+}
+
+static void 
+afs_DCSizeInit(void) 
+{
+    afs_blocksUsed_0 = afs_blocksUsed_1 = afs_blocksUsed_2 = 0;
+}
+
+static afs_int32
+afs_DCWhichBucket(afs_int32 phase, afs_int32 bucket) 
+{
+    if (!splitdcache) 
+	return 0;
+
+    afs_pct1 = afs_blocksUsed_1 / (afs_cacheBlocks / 100);
+    afs_pct2 = afs_blocksUsed_2 / (afs_cacheBlocks / 100);
+
+    /* Short cut: if we don't know about it, try to kill it */
+    if (phase < 2 && afs_blocksUsed_0) 
+	return 0;
+    
+    if (afs_pct1 > afs_tpct1) 
+	return 1;
+    if (afs_pct2 > afs_tpct2)
+	return 2;
+    return 0; /* unlikely */
+}
+
+
 /*
  * afs_StoreWarn
  *
@@ -223,16 +346,14 @@ afs_CacheTruncateDaemon(void)
     u_int counter;
     u_int cb_lowat;
     u_int dc_hiwat =
-	(100 - CM_DCACHECOUNTFREEPCT +
-	 CM_DCACHEEXTRAPCT) * afs_cacheFiles / 100;
+	PERCENT((100 - CM_DCACHECOUNTFREEPCT + CM_DCACHEEXTRAPCT), afs_cacheFiles);
     afs_min_cache =
 	(((10 * AFS_CHUNKSIZE(0)) + afs_fsfragsize) & ~afs_fsfragsize) >> 10;
 
     osi_GetuTime(&CTD_stats.CTD_afterSleep);
     afs_TruncateDaemonRunning = 1;
     while (1) {
-	cb_lowat = ((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT)
-		    * afs_cacheBlocks) / 100;
+	cb_lowat = PERCENT((CM_DCACHESPACEFREEPCT - CM_DCACHEEXTRAPCT), afs_cacheBlocks);
 	MObtainWriteLock(&afs_xdcache, 266);
 	if (afs_CacheTooFull) {
 	    int space_needed, slots_needed;
@@ -242,7 +363,7 @@ afs_CacheTruncateDaemon(void)
 		    afs_blocksUsed - afs_blocksDiscarded - cb_lowat;
 		slots_needed =
 		    dc_hiwat - afs_freeDCCount - afs_discardDCCount;
-		afs_GetDownD(slots_needed, &space_needed);
+		afs_GetDownD(slots_needed, &space_needed, 0);
 		if ((space_needed <= 0) && (slots_needed <= 0)) {
 		    break;
 		}
@@ -337,6 +458,7 @@ afs_AdjustSize(register struct dcache *adc, register afs_int32 newSize)
     if (!newSize)
 	adc->validPos = 0;
     newSize = ((newSize + afs_fsfragsize) ^ afs_fsfragsize) >> 10;	/* round up */
+    afs_DCAdjustSize(adc, oldSize, newSize);
     if (newSize > oldSize) {
 	/* We're growing the file, wakeup the daemon */
 	afs_MaybeWakeupTruncateDaemon();
@@ -373,7 +495,7 @@ afs_AdjustSize(register struct dcache *adc, register afs_int32 newSize)
 
 #define	MAXATONCE   16		/* max we can obtain at once */
 static void
-afs_GetDownD(int anumber, int *aneedSpace)
+afs_GetDownD(int anumber, int *aneedSpace, afs_int32 buckethint)
 {
 
     struct dcache *tdc;
@@ -389,6 +511,7 @@ afs_GetDownD(int anumber, int *aneedSpace)
     afs_hyper_t maxVictimTime;	/* youngest (largest LRU time) victim */
     afs_uint32 maxVictimPtr;	/* where it is */
     int discard;
+    int curbucket;
 
     AFS_STATCNT(afs_GetDownD);
     if (CheckLock(&afs_xdcache) != -1)
@@ -405,12 +528,20 @@ afs_GetDownD(int anumber, int *aneedSpace)
     if (anumber > MAXATONCE)
 	anumber = MAXATONCE;	/* all we can do */
 
+    /* rewrite so phases include a better eligiblity for gc test*/
     /*
      * The phase variable manages reclaims.  Set to 0, the first pass,
-     * we don't reclaim active entries.  Set to 1, we reclaim even active
-     * ones.
+     * we don't reclaim active entries, or other than target bucket.  
+     * Set to 1, we reclaim even active ones in target bucket.
+     * Set to 2, we reclaim any inactive one.
+     * Set to 3, we reclaim even active ones.
      */
-    phase = 0;
+    if (splitdcache) {
+	phase = 0;
+    } else {
+	phase = 4;
+    }
+
     for (i = 0; i < afs_cacheFiles; i++)
 	/* turn off all flags */
 	afs_indexFlags[i] &= ~IFFlag;
@@ -419,6 +550,7 @@ afs_GetDownD(int anumber, int *aneedSpace)
 	/* find oldest entries for reclamation */
 	maxVictimPtr = victimPtr = 0;
 	hzero(maxVictimTime);
+	curbucket = afs_DCWhichBucket(phase, buckethint);
 	/* select victims from access time array */
 	for (i = 0; i < afs_cacheFiles; i++) {
 	    if (afs_indexFlags[i] & (IFDataMod | IFFree | IFDiscarded)) {
@@ -426,6 +558,11 @@ afs_GetDownD(int anumber, int *aneedSpace)
 		continue;
 	    }
 	    tdc = afs_indexTable[i];
+	    if (tdc && (curbucket != tdc->bucket) && (phase < 4))
+	    {
+		/* Wrong bucket; can't use it! */
+	        continue;
+	    }
 	    if (tdc && (tdc->refCount != 0)) {
 		/* Referenced; can't use it! */
 		continue;
@@ -507,12 +644,12 @@ afs_GetDownD(int anumber, int *aneedSpace)
 		if (tvc) {
 		    tchunkoffset = AFS_CHUNKTOBASE(tdc->f.chunk);
 		    chunkFlags = afs_indexFlags[tdc->index];
-		    if (phase == 0 && osi_Active(tvc))
-			skip = 1;
-		    if (phase > 0 && osi_Active(tvc)
-			&& (tvc->states & CDCLock)
-			&& (chunkFlags & IFAnyPages))
-			skip = 1;
+		    if (((phase & 1) == 0) && osi_Active(tvc))
+                        skip = 1;
+		    if (((phase & 1) == 1) && osi_Active(tvc)
+                        && (tvc->states & CDCLock)
+                        && (chunkFlags & IFAnyPages))
+                        skip = 1;
 		    if (chunkFlags & IFDataMod)
 			skip = 1;
 		    afs_Trace4(afs_iclSetp, CM_TRACE_GETDOWND,
@@ -583,7 +720,7 @@ afs_GetDownD(int anumber, int *aneedSpace)
 			MReleaseWriteLock(&afs_xdcache);
 		    }
 
-		    AFS_FAST_RELE(tvc);
+		    afs_PutVCache(tvc); /*XXX was AFS_FAST_RELE?*/
 		    MObtainWriteLock(&afs_xdcache, 528);
 		    if (afs_indexFlags[tdc->index] &
 			(IFDataMod | IFDirtyPages | IFAnyPages))
@@ -644,18 +781,18 @@ afs_GetDownD(int anumber, int *aneedSpace)
 	    afs_PutDCache(tdc);
 	}
 
-	if (phase == 0) {
+	if (phase < 5) {
 	    /* Phase is 0 and no one was found, so try phase 1 (ignore
 	     * osi_Active flag) */
 	    if (j == 0) {
-		phase = 1;
+		phase++;
 		for (i = 0; i < afs_cacheFiles; i++)
 		    /* turn off all flags */
 		    afs_indexFlags[i] &= ~IFFlag;
 	    }
 	} else {
-	    /* found no one in phase 1, we're hosed */
-	    if (victimPtr == 0)
+	    /* found no one in phases 0-5, we're hosed */
+	    if (j == 0)
 		break;
 	}
     }				/* big while loop */
@@ -777,7 +914,7 @@ afs_FlushDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if (afs_blocksUsed <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -810,7 +947,7 @@ afs_FreeDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -860,7 +997,7 @@ afs_DiscardDCache(register struct dcache *adc)
 
     if (afs_WaitForCacheDrain) {
 	if ((afs_blocksUsed - afs_blocksDiscarded) <=
-	    (CM_CACHESIZEDRAINEDPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_CACHESIZEDRAINEDPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 0;
 	    afs_osi_Wakeup(&afs_WaitForCacheDrain);
 	}
@@ -913,6 +1050,8 @@ afs_FreeDiscardedDCache(void)
     afs_CFileTruncate(tfile, 0);
     afs_CFileClose(tfile);
     afs_AdjustSize(tdc, 0);
+    tdc->f.states &= ~(DRO|DBackup|DRW);
+    afs_DCMoveBucket(tdc, 0, 0);
 
     /*
      * Free the element we just truncated
@@ -943,7 +1082,7 @@ afs_MaybeFreeDiscardedDCache(void)
 
     while (afs_blocksDiscarded
 	   && (afs_blocksUsed >
-	       (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+	       PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	afs_FreeDiscardedDCache();
     }
     return 0;
@@ -1200,6 +1339,7 @@ afs_FindDCache(register struct vcache *avc, afs_size_t abyte)
 		break;		/* leaving refCount high for caller */
 	    }
 	    afs_PutDCache(tdc);
+	    tdc = NULL;
 	}
 	index = afs_dcnextTbl[index];
     }
@@ -1687,7 +1827,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 		while (1) {
 		    if (!setLocks)
 			avc->states |= CDCLock;
-		    afs_GetDownD(5, (int *)0);	/* just need slots */
+		    /* just need slots */
+		    afs_GetDownD(5, (int *)0, afs_DCGetBucket(avc));
 		    if (!setLocks)
 			avc->states &= ~CDCLock;
 		    if (afs_discardDCList != NULLIDX
@@ -1728,6 +1869,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 		size =
 		    ((tdc->f.chunkBytes +
 		      afs_fsfragsize) ^ afs_fsfragsize) >> 10;
+		tdc->f.states &= ~(DRO|DBackup|DRW);
+		afs_DCMoveBucket(tdc, size, 0);
 		afs_blocksDiscarded -= size;
 		afs_stats_cmperf.cacheBlocksDiscarded = afs_blocksDiscarded;
 		if (aflags & 2) {
@@ -1752,6 +1895,13 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	     */
 	    afs_indexFlags[tdc->index] &= ~(IFDirtyPages | IFAnyPages);
 	    tdc->f.fid = avc->fid;
+	    if (avc->states & CRO) 
+		tdc->f.states = DRO;
+	    else if (avc->states & CBackup) 
+		tdc->f.states = DBackup;
+	    else 
+		tdc->f.states = DRW;
+	    afs_DCMoveBucket(tdc, 0, afs_DCGetBucket(avc));
 	    afs_indexUnique[tdc->index] = tdc->f.fid.Fid.Unique;
 	    hones(tdc->f.versionNo);	/* invalid value */
 	    tdc->f.chunk = chunk;
@@ -1771,7 +1921,6 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    afs_dvhashTbl[i] = tdc->index;
 	    tdc->dflags = DFEntryMod;
 	    tdc->mflags = 0;
-	    tdc->f.states = 0;
 	    afs_MaybeWakeupTruncateDaemon();
 	    MReleaseWriteLock(&afs_xdcache);
 	    ConvertWToSLock(&tdc->lock);
@@ -1967,7 +2116,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	/* Sleep here when cache needs to be drained. */
 	if (setLocks && !slowPass
 	    && (afs_blocksUsed >
-		(CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+		PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	    /* Make sure truncate daemon is running */
 	    afs_MaybeWakeupTruncateDaemon();
 	    ObtainWriteLock(&tdc->tlock, 614);
@@ -1976,7 +2125,7 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    ReleaseWriteLock(&tdc->lock);
 	    ReleaseReadLock(&avc->lock);
 	    while ((afs_blocksUsed - afs_blocksDiscarded) >
-		   (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+		   PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 		afs_WaitForCacheDrain = 1;
 		afs_osi_Sleep(&afs_WaitForCacheDrain);
 	    }
@@ -2382,6 +2531,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	    if (vType(avc) == VDIR) {
 		DZap(tdc);
 	    }
+	    tdc->f.states &= ~(DRO|DBackup|DRW);
+	    afs_DCMoveBucket(tdc, 0, 0);
 	    ReleaseWriteLock(&tdc->lock);
 	    afs_PutDCache(tdc);
 	    ObtainWriteLock(&afs_xcbhash, 454);
@@ -2395,6 +2546,8 @@ afs_GetDCache(register struct vcache *avc, afs_size_t abyte,
 	     * avc->lock(W); assert(!setLocks || slowPass)
 	     */
 	    osi_Assert(!setLocks || slowPass);
+	    tdc->f.states &= ~(DRO|DBackup|DRW);
+	    afs_DCMoveBucket(tdc, 0, 0);
 	    tdc = NULL;
 	    goto done;
 	}
@@ -2804,6 +2957,18 @@ afs_UFSGetDSlot(register afs_int32 aslot, register struct dcache *tmpdc)
 #endif
 	lasterrtime = osi_Time();
 	afs_indexUnique[aslot] = tdc->f.fid.Fid.Unique;
+	tdc->f.states &= ~(DRO|DBackup|DRW);
+	afs_DCMoveBucket(tdc, 0, 0);
+    } else {
+	if (&tdc->f != 0) {
+	    if (tdc->f.states & DRO) {
+		afs_DCMoveBucket(tdc, 0, 2);
+	    } else if (tdc->f.states & DBackup) {
+		afs_DCMoveBucket(tdc, 0, 1);
+	    } else {
+		afs_DCMoveBucket(tdc, 0, 1); 
+	    }
+	} 
     }
     tdc->refCount = 1;
     tdc->index = aslot;
@@ -3027,6 +3192,8 @@ afs_InitCacheFile(char *afile, ino_t ainode)
 	tdc->f.fid.Fid.Volume = 0;	/* not in the hash table */
 	if (tstat.size != 0)
 	    osi_UFSTruncate(tfile, 0);
+	tdc->f.states &= ~(DRO|DBackup|DRW);
+	afs_DCMoveBucket(tdc, 0, 0);
 	/* put entry in free cache slot list */
 	afs_dvnextTbl[tdc->index] = afs_freeDCList;
 	afs_freeDCList = index;
@@ -3201,6 +3368,10 @@ afs_dcacheInit(int afiles, int ablocks, int aDentries, int achunk, int aflags)
 
     afs_dcentries = aDentries;
     afs_blocksUsed = 0;
+    afs_stats_cmperf.cacheBucket0_Discarded = 
+	afs_stats_cmperf.cacheBucket1_Discarded = 
+	afs_stats_cmperf.cacheBucket2_Discarded = 0;
+    afs_DCSizeInit();
     QInit(&afs_DLRU);
 }
 
@@ -3241,6 +3412,9 @@ shutdown_dcache(void)
     afs_osi_Free(afs_dchashTbl, afs_dhashsize * sizeof(afs_int32));
 
     afs_blocksUsed = afs_dcentries = 0;
+    afs_stats_cmperf.cacheBucket0_Discarded = 
+	afs_stats_cmperf.cacheBucket1_Discarded = 
+	afs_stats_cmperf.cacheBucket2_Discarded = 0;
     hzero(afs_indexCounter);
 
     afs_freeDCCount = 0;
