@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/dir/dir.c,v 1.21.2.7 2007/10/30 15:23:52 shadow Exp $");
+    ("$Header: /cvs/openafs/src/dir/dir.c,v 1.24 2005/10/13 15:12:12 shadow Exp $");
 
 #ifdef KERNEL
 #if !defined(UKERNEL)
@@ -100,7 +100,13 @@ extern void *DNew();
 #include <netinet/in.h>
 #endif
 
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #endif /* KERNEL */
 
 afs_int32 DErrno;
@@ -472,7 +478,7 @@ DirHash(register char *string)
 {
     /* Hash a string to a number between 0 and NHASHENT. */
     register unsigned char tc;
-    unsigned int hval;
+    register int hval;
     register int tval;
     hval = 0;
     while ((tc = (*string++))) {
@@ -482,7 +488,7 @@ DirHash(register char *string)
     tval = hval & (NHASHENT - 1);
     if (tval == 0)
 	return tval;
-    else if (hval >= 1<<31)
+    else if (hval < 0)
 	tval = NHASHENT - tval;
     return tval;
 }
